/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/*
 * Inlined classifier.
 */

#include <clack/c/packet.h>
#include <clack/c/packet_ip.h>
#include <clack/c/packet_eth.h>
#include <clack/c/if_arp.h>
#include <arpa/inet.h>

/*
 * Imported
 */
extern clack_error_t	push_toarpr(clack_packet_t *);
extern clack_error_t	push_toarpq(clack_packet_t *);
extern clack_error_t	push_torouter(clack_packet_t *);
extern clack_error_t	push_tofail(clack_packet_t *);

/*
 * Exported
 */
clack_error_t		push(clack_packet_t *p);

clack_error_t
push(clack_packet_t *p)
{
#if 1
	/*
	 * Smoosh them all together the simple minded way.
	 */
	struct ether_header *e  = (struct ether_header *) clack_packet_data(p);
	struct ether_arp    *ea = (struct ether_arp *) (e + 1);
	clack_size_t	    length = clack_packet_length(p);

	if ((length >= sizeof(*e) + sizeof(struct ether_arp)) &&
	    ntohs(e->ether_type)     == ETHERTYPE_ARP &&
	    ntohs(ea->ea_hdr.ar_hrd) == ARPHRD_ETHER &&
	    ntohs(ea->ea_hdr.ar_pro) == ETHERTYPE_IP &&
	    ntohs(ea->ea_hdr.ar_op)  == ARPOP_REQUEST) {
		return push_toarpr(p);
	}
	if ((length >= sizeof(*e) + sizeof(struct ether_arp)) &&
	    ntohs(e->ether_type)     == ETHERTYPE_ARP &&
	    ntohs(ea->ea_hdr.ar_hrd) == ARPHRD_ETHER &&
	    ntohs(ea->ea_hdr.ar_pro) == ETHERTYPE_IP &&
	    ntohs(ea->ea_hdr.ar_op)  == ARPOP_REPLY) {
		return push_toarpq(p);
	}
	if (ntohs(e->ether_type) == ETHERTYPE_IP) {
		return push_torouter(p);
	}
	return push_tofail(p);
#else
	/*
	 * The Click fastclassifier compiler generated this.
	 */
	unsigned *data = (unsigned *) clack_packet_data(p);

 step_0:
	if ((data[3] & 65535U) != 1544U)
		goto step_3;
 step_1:
	if ((data[5] & 65535U) == 256U) {
		return push_toarpr(p);
	}
 step_2:
	if ((data[5] & 65535U) == 512U) {
		return push_toarpq(p);
	}
	return push_tofail(p);
 step_3:
	if ((data[3] & 65535U) == 8U) {
		return push_torouter(p);
	}
	return push_tofail(p);
#endif
}
