/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <clack/c/packet_icmp.h>
#include <arpa/inet.h> 
#include <clack/c/icmp.h>

void add_icmp_hdr_echo(clack_packet_t *p, clack_u8_t type, clack_u8_t code, clack_u16_t id, clack_u16_t sequence)
{
        struct icmphdr *icmp_hdr = clack_packet_UNSHIFT(p, struct icmphdr);

        icmp_hdr->type = type;
        icmp_hdr->code = code;
        icmp_hdr->checksum = 0;
        icmp_hdr->un.echo.id = id;  // already in network order
        icmp_hdr->un.echo.sequence = sequence; // ditto
        icmp_hdr->checksum = in_cksum((unsigned char *) icmp_hdr, clack_packet_length(p));
}

void add_icmp_hdr_gateway(clack_packet_t *p, clack_u8_t type, clack_u8_t code, clack_u32_t gateway)
{
        struct icmphdr *icmp_hdr = clack_packet_UNSHIFT(p, struct icmphdr);

        icmp_hdr->type = type;
        icmp_hdr->code = code;
        icmp_hdr->checksum = 0;
        icmp_hdr->un.gateway = gateway;
        icmp_hdr->checksum = in_cksum((unsigned char *) icmp_hdr, clack_packet_length(p));
}

void add_icmp_hdr_frag(clack_packet_t *p, clack_u8_t type, clack_u8_t code, clack_u16_t mtu)
{
        struct icmphdr *icmp_hdr = clack_packet_UNSHIFT(p, struct icmphdr);

        icmp_hdr->type = type;
        icmp_hdr->code = code;
        icmp_hdr->checksum = 0;
        icmp_hdr->un.frag.__unused = 0;
        icmp_hdr->un.frag.mtu = htons(mtu);
        icmp_hdr->checksum = in_cksum((unsigned char *) icmp_hdr, clack_packet_length(p));
}

